@@ -189,10 +189,10 @@ class DownloadListItem : public QListViewItem
    {
       transferpopupmenu->insertItem(i18n("Clear Errors and Finished Transfers"),
 	    this,SLOT(popupClear())) ;
-#ifdef DEVEL_CODE    
+//#ifdef DEVEL_CODE    
       transferpopupmenu->insertSeparator();
       transferpopupmenu->insertItem(i18n("Play/Stop"),this,SLOT(popupDownloadPlay())) ;
-#endif
+//#endif
       transferpopupmenu->insertSeparator();
       transferpopupmenu->insertItem(i18n("Ping user"),this,SLOT(popupPing()));
       transferpopupmenu->insertItem(i18n("Whois user"),this,SLOT(popupWhois()));
@@ -310,10 +310,26 @@ void KNAP_TransferPanel::popupClear()
 void KNAP_TransferPanel::popupDownloadPlay()
 {
    QListViewItem *selecteditem=transferlistview->currentItem();
+   
+   SongInfo *song;
+   song=transfers.first();
 
-   if(selecteditem) {
-      // TODO ********* get the filename associated with this transfer
-      emit play(selecteditem->text(DLV_FILE));
+   if(selecteditem) 
+   {
+
+      while(song != 0)
+      {
+        if(song->internal_id.findRev(selecteditem->text(DLV_XTRA))==0)
+        {
+
+         cerr << "KNAP_TransferPanel::popupDownloadPlay(): "
+	     << song->fullpathname << endl;
+	 return;
+        }
+
+        song=transfers.next();
+      } 
+     //emit sendcommand(QString("/play ") + selecteditem->text(DLV_FILE));
    }
 }
 
@@ -324,7 +340,7 @@ void KNAP_TransferPanel::popupDownloadTerminate()
 
    if(it) 
    {
-      emit terminate(it);
+      //emit terminate(it);
       // cancel the dload
 
       //if(sg->internal_id.findRev(it->text(DLV_XTRA))==0) 
@@ -345,7 +361,7 @@ void KNAP_TransferPanel::popupDownloadTerminateAndDelete()
    QListViewItem *selecteditem=transferlistview->currentItem();
 
    if(selecteditem) {
-      emit terminate_and_delete(selecteditem);
+      //emit terminate_and_delete(selecteditem);
    }
 }
 
@@ -450,9 +466,10 @@ void KNAP_TransferPanel::addDownloadItem(const char *user,const char *file)
 	 //song->md5);
 	 song->internal_id);
 
-
+   song->fullpathname = file;
+   
 	 // we own the song now and will free it
-	 transfers.append(song);
+   transfers.append(song);
 }
 
 bool KNAP_TransferPanel::stopDownload(SongInfo *si,bool findbyid,bool naperror)
