@@ -43,6 +43,7 @@
 #include "knap_whois.h"
 #include "knap_napigatordlg.h"
 #include "knap_connectdlg.h"
+#include "knap_ipaddressdlg.h"
 
 #include "knap_createchandlg.h"
 #include "knap_chanlist.h"
@@ -136,7 +137,10 @@ void Knapster2::setupMenus()
    connmenu = new KAction(i18n("&Connect"),0,this,SLOT ( menuConnect() ),
 	 actionCollection(),"login");
 
-   new KAction(i18n("Connect via &Napigator"),0,this,SLOT ( menuNapigator() ),
+   new KAction(i18n("Connect &To..."),0,this,SLOT ( menuConnectTo() ),
+	 actionCollection(),"login_to");
+
+   new KAction(i18n("Connect via &Napigator..."),0,this,SLOT ( menuNapigator() ),
 	 actionCollection(),"login_napigator");
 
    KStdAction::quit(kapp, SLOT(closeAllWindows()), actionCollection()); 
@@ -155,7 +159,7 @@ void Knapster2::setupMenus()
    new KAction(i18n("&Channels..."),0,this,SLOT ( menuChannels() ),
 	 actionCollection(),"show_chanlist");
 
-   new KAction(i18n("Create &New channel"),0,this,SLOT ( menuCreateChannel() ),
+   new KAction(i18n("Create &New channel..."),0,this,SLOT ( menuCreateChannel() ),
 	 actionCollection(),"join_chan");
 
    createGUI(); // read the resource file
@@ -228,6 +232,16 @@ void Knapster2::setupTabs()
 #endif
    setCentralWidget(MainFrame);
 
+}
+void Knapster2::menuConnectTo()
+{
+   KNAP_IPAddressDlg addr(this,"Connect-to");
+   addr.show();
+   
+   QString server = addr.getIPAddress();
+   
+   if(server.length()>0) menuRecent("http://" + server);
+
 }
 void Knapster2::menuConnect()
 {
