@@ -87,7 +87,8 @@ const char *NapsterConnection::getLastErrorStr() const
 {
    incominglist.setAutoDelete(true);
 
-   
+   _block_pos = 0;
+
    _connected=false;
    _havehost=false;
 
@@ -114,41 +115,6 @@ NapsterConnection::~NapsterConnection()
    close();
 }
 
-// assumes getBestHost was called
-/*
-   bool NapsterConnection::open(int port)
-   {
-   close();
-
-   _lasterror=NAP_OK;
-
-   _listenerport=port; // save the listen port for doing logins
-
-   setupListener(port);
-
-   cerr << "napsterconnection::open(): " << _napsteraddr << " " << _napsterport <<endl;
-
-   _napster = new KSocket(_napsteraddr,_napsterport); // timeout
-
-   if(!_napster || _napster->socket()==-1)
-   {
-   delete _napster;
-   _napster=0;
-#ifdef DEBUG_1
-cerr<<"Can't connect!\n"<<flush;
-#endif
-  _lasterror=NAP_NO_CONNECT;
-  return(false);
-}
-#ifdef DEBUG_1
-cerr<<"connected to "<< getIPAddressString() << " ("<<_napster->socket()<<")"<<endl;
-#endif
-
-_connected = true;
-
-return true;
-}
-*/
 // assumes getBestHost was called
 bool NapsterConnection::open(int port)
 {
@@ -520,7 +486,7 @@ bool NapsterConnection::login(const char *_username,const char *password,
 
       NAPBLOCKPTR ptr;
 
-      ptr=readBlock();
+      ptr=readBlock(true);
 
       cout<<"resp: ["<<ptr->type<<"] "
 	 <<" ["<<ptr->size<<"] "<<ptr->data<<endl<<flush;
@@ -596,6 +562,7 @@ bool NapsterConnection::login(const char *_username,const char *password,
       }
    }
    // wow!! - we finally did it
+   cerr << "Connected !!!!!!!!!!!\n";
    _connected=true;
 
    return true;
@@ -604,23 +571,11 @@ bool NapsterConnection::login(const char *_username,const char *password,
 
 QString NapsterConnection::getIPAddressString() const
 {
-   //  QString s="";
-   //  struct in_addr in;
-
-   //  if(_napster)
-   //  {
-   //     in.s_addr = _napster->ipv4_addr();
-   //     s=inet_ntoa(in);
-   //  }
-   //  return(s);
    return _napsteraddr;
 }
 
 int NapsterConnection::getIPPort() const
 {
-   //return ntohs(_sockaddr.sin_port);
-   // _sockaddr.sin_port = htons
-   //_napster->
    return _napsterport;
 }
 
@@ -631,10 +586,6 @@ int NapsterConnection::readComm(char *xbuffer,int size)
 {
    int s=0;
 
-#ifdef KDE_INTERACTION
-//   kapp->processEvents(1000);
-#endif
-
    _lasterror=NAP_OK;
 
    if(_napster==0) 
@@ -650,10 +601,6 @@ int NapsterConnection::readComm(char *xbuffer,int size)
    // just whatever we can get??
    s=recv(_napster->socket(),xbuffer,size,0);
 
-   //#ifdef DEBUG_1
-   //   if(s>=0) xbuffer[s]='\0';
-   //   cerr << "readCom(): [" << s << "]: " << xbuffer <<endl;
-   //#endif  
 
    return(s);
 }
@@ -663,10 +610,6 @@ int NapsterConnection::writeComm(const char *astring,int size)
 
    _lasterror=NAP_OK;
 
-#ifdef KDE_INTERACTION
-//   kapp->processEvents(1000);
-#endif
-
    if(_napster==0) 
    {
       _lasterror=NAP_NO_CONNECT;
@@ -677,121 +620,159 @@ int NapsterConnection::writeComm(const char *astring,int size)
    return s;
 }
 
+
 NAPBLOCKPTR NapsterConnection::readBlock(bool waitfordata)
 {
-   int s;
-   NAPBLOCKPTR ptr;
+  int s;
+  NAPBLOCKPTR ptr;
+  static NapBlock error_block = { 0, NAP_ERROR };
+    
+  ptr=(NAPBLOCKPTR)_buffer;
+       
+//  ptr->type=NAP_ERROR;
+//  ptr->size=0;
+	     
+  if(_napster==0)		     
+  {			      
+     _lasterror=NAP_NO_CONNECT;
+			       
+     return &error_block;				       
+  }  
+  
+  if(waitfordata)
+  {
+      _block_pos = 0; // there is no partial data when have to wait for it
 
-   ptr=(NAPBLOCKPTR)_buffer;
-
-   ptr->type=NAP_ERROR;
-   ptr->size=0;
-
-   if(_napster==0) 
-   {
-      _lasterror=NAP_NO_CONNECT;
-
-      return ptr;
-   }
-   // --- test that reads will get valid data
-
-   // TODO - force it to wait for data if waitfordata is true
-   if(waitfordata)
-   {
       fd_set fdsr;
-   //   struct timeval tm;
-
-   //   tm.tv_sec=0;
-   //   tm.tv_usec=500;
 
       FD_ZERO(&fdsr);
       FD_SET(_napster->socket(),&fdsr);
       select(_napster->socket()+1,&fdsr,NULL,NULL,NULL);
-   }
 
-   if(!waitfordata)
-   {
-      //#ifdef DEBUG_1
-      //      cerr << "in wait for data\n";
-      //#endif  
-      s = recv(_napster->socket(),_buffer,MAX_NAP_BLOCK,MSG_PEEK);
-      if(s<4)
+      if((s = readComm(_buffer,4))<=0)  // error or no data
       {
-	 /* not even the start of the header yet */
-	 ptr->size=0;
-	 ptr->type=NAP_ERROR;
-
-	 _lasterror = NAP_NO_READ;
-	 return ptr;
+         _lasterror=NAP_NO_READ;
+         cout<<"read err1: "<<s<<endl<<flush;
+        return &error_block;
       }
+
       ptr->size=NAP_TO_SHORT(ptr->size); // make sure its in the right endian
+      ptr->type=NAP_TO_SHORT(ptr->type);
 
-      if(ptr->size<MAX_NAP_BLOCK) // TODO valid to be > ??
+      if((signed short int)ptr->size < 0 || ptr->size > MAX_NAP_BLOCK) 
       {
-	 //       s = ptr->size;
-	 //       if(recv(_napster,_buffer,4+s,MSG_PEEK)<(4+s))
-	 if((ptr->size + 4) > s)
-	 {
-	    // not enough to read yet
-
-	    ptr->size=0;
-	    ptr->type=NAP_ERROR;
-
-	    _lasterror = NAP_NO_READ;
-	    return ptr;
-	 }
+        _lasterror=NAP_BAD_RECV_DATA;
+        cout<<"read err2: "<<ptr->size<<endl<<flush;
+        return &error_block;
       }
-   }
-
-   //--------------
-   // now do the real read
-   if((s = readComm(_buffer,4))<=0)  // error or no data
-   {
-      _lasterror=NAP_NO_READ;
-#ifdef DEBUG_1
-      cout<<"read err1: "<<s<<endl<<flush;
-#endif
-      return ptr;
-   }
-
-   ptr->size=NAP_TO_SHORT(ptr->size); // make sure its in the right endian
-   ptr->type=NAP_TO_SHORT(ptr->type);
-
-   if((short int)ptr->size < 0) {
-      //cout<<"size error\n";
-      ptr->type=NAP_ERROR;
-      _lasterror=NAP_BAD_RECV_DATA;
-#ifdef DEBUG_1
-      cout<<"read err2: "<<ptr->size<<endl<<flush;
-#endif
-      ptr->size=0;
-   }
-   else if(ptr->size<MAX_NAP_BLOCK) 
-   {      // ******** may be > 1024 ?????????????????
-
+      
       s=readComm(ptr->data,ptr->size );
 
       ptr->data[s]='\0';
+      ptr->size = s;
 
       _lasterror=NAP_OK;
-#ifdef DEBUG_2
-      cout<<"NapsterConnection::readBlock(): ["
+#ifdef DEBUG_1
+      cout<<"NapsterConnection::readBlock(): (waitfordata) ["
 	 << ptr->type << "]: (" << s <<")bytes "
 	 << ptr->data << endl << flush;
-#endif
+#endif      
       ptr->size=s; //********************
-   }
-   else 
-   {
-#ifdef DEBUG_1
-      cout<<"Someting bad happened in readBlock\n";
-#endif
-      ptr->type=NAP_ERROR;
-      _lasterror=NAP_BAD_RECV_DATA;
-      ptr->size=0;
-   }
-   return ptr;
+ 
+      return ptr;
+  }
+  else // (!waitfordata)
+  {
+     if(_block_pos == 0) // none read yet
+     {
+       _block_pos = readComm(_buffer,MIN_NAP_BLOCK);
+       if(_block_pos == MIN_NAP_BLOCK) 
+       {
+	  ptr->size=NAP_TO_SHORT(ptr->size); 
+          ptr->type=NAP_TO_SHORT(ptr->type);
+       }
+       // make sure its in the right endian
+       if(_block_pos<0)
+       {
+	  _block_pos = 0;
+	  
+          _lasterror=NAP_BAD_RECV_DATA;
+	  return &error_block;
+ 
+       }
+       else if(_block_pos==0)
+       {
+	  
+          _lasterror = NAP_NO_READ;
+	  return &error_block;
+       }
+     }
+     else if(_block_pos < MIN_NAP_BLOCK) // try to read rest of start of block
+     {
+       s = readComm(&_buffer[_block_pos],MIN_NAP_BLOCK - _block_pos);
+       if(s > 0) _block_pos += s;
+       
+       if(_block_pos == MIN_NAP_BLOCK) 
+       {
+	  ptr->size=NAP_TO_SHORT(ptr->size); 
+          ptr->type=NAP_TO_SHORT(ptr->type);
+       }
+       // make sure its in the right endian
+     }
+     else // try to read rest of block in - we already have header part
+     {
+       if(ptr->size > MAX_NAP_BLOCK || (signed short)ptr->size < 0) 
+	  // TODO valid to be > ??
+       {
+	  _block_pos = 0;
+	  
+          //_lasterror = NAP_NO_READ;
+          _lasterror=NAP_BAD_RECV_DATA;
+	  return &error_block;
+       }
+
+       s = readComm(&_buffer[_block_pos],ptr->size + MIN_NAP_BLOCK - _block_pos);
+       if(s > 0) _block_pos += s;
+      
+     }
+
+     if((signed short)ptr->size < 0)
+     {
+        _block_pos = 0;
+	   
+	//_lasterror = NAP_NO_READ;
+	_lasterror=NAP_BAD_RECV_DATA;
+	return &error_block; 
+     }
+     
+     // have  a block now?
+     if(_block_pos >= MIN_NAP_BLOCK && ((ptr->size + MIN_NAP_BLOCK) <= _block_pos))
+     {
+
+        _buffer[_block_pos]='\0';
+
+#ifdef DEBUG_1	
+	cout<<"*NapsterConnection::readBlock(): " << _block_pos << " ["
+	 << ptr->type << "]: (" << ptr->size <<")bytes "
+	 << ptr->data << endl << flush;
+#endif 
+
+
+        _block_pos = 0;
+        _lasterror=NAP_OK;
+        return ptr;
+     }
+     else
+     {   // not enough read yet
+        _lasterror = NAP_NO_READ;
+	return &error_block; 
+     }
+  } // end if ! waitfordata
+
+  _lasterror = NAP_NO_READ;  
+  return &error_block;
 }
+     
 
 int NapsterConnection::writeBlock(int type,const QString &data)
 {
@@ -814,29 +795,6 @@ int NapsterConnection::writeBlock(int type,const QString &data)
 
    return s;
 }
-/*
-   int NapsterConnection::writeBlock(int type,const char *data,int size)
-   {
-   NapBlock blk;
-
-   int s;
-
-// fix for the endianess problem
-blk.size=SHORT_TO_NAP(size);
-blk.type=SHORT_TO_NAP(type);
-
-s=writeComm((char *)&blk,4);
-if(s!=-1) 
-{
-// send the actual data
-if(size>0)
-s=writeComm(data,size);
-}
-else  _lasterror=NAP_NO_WRITE;
-
-return s;
-}
- */
 // ---------------------------------------------------------------------
 bool NapsterConnection::setupListener(int _port)
 {
@@ -907,19 +865,15 @@ bool NapsterConnection::share(const SongInfo *s)
       + " " + QString().setNum(s->seconds);
    //  str=str.stripWhiteSpace();
 
-#ifdef DEBUG_1
+//#ifdef DEBUG_1
    cerr<<"sending shared: "<<str<<endl<<flush;
-#endif
+//#endif
    // since a lot of shares can be sent at a time, make sure
    // we have sent every thing in the send buffer before we send 
    // more data
    cerr << "Wait send...";
 
    fd_set fdsw;
-//   struct timeval tm;
-
-//   tm.tv_sec=0;
-//   tm.tv_usec=500;
 
    FD_ZERO(&fdsw);
 
@@ -939,8 +893,6 @@ void NapsterConnection::sendSearch(const char *name)
       return;
    }
 
-   //   int l=strlen(name);
-
    if(writeBlock(NAP_SEARCH_REQ,name)==-1)
       _lasterror=NAP_NO_WRITE;
 } 
